// Prototype MIDI Manipulator
// Reads in a MIDI file, manipulates the timing according to a uniformly distributed
// random number generator operating on the range of [-10ms,10ms], and then 
// writes to a new MIDI file,

// Jacob Peck - Team La-a
// CSC 380 - Early
// 29 September 2011


// compilation: javac -cp path/to/jfugue.jar:. MIDIManip.java
// usage: java -cp path/to/jfugue.jar:. MIDIManip some-input-midi-file.midi

import org.jfugue.*; // for JFugue stuff
import java.io.*; // for File I/O stuff
import java.util.*; // for Scanner and Random

public class MIDIManip {
  public static void main(String[] args) throws Exception {
    String filename = args[0]; // filename is first command line argument
    Player player = new Player(); // player, to perform loads/saves
    Pattern pattern = player.loadMidi(new File(filename)); // loads a midi file into a JFugue pattern
    String musicstring = pattern.getMusicString(); // gets the string representation of the file
    System.out.println("Original: " + musicstring); // prints it out (for comparison sake)
    Scanner sc = new Scanner(musicstring); // for parsing
    String newmusicstring = ""; // to hold the manipulated file
    Random rng = new Random(); // random number generator, beeyotch.
    while(sc.hasNext()) { // run on the entire string
      String token = sc.next(); // next token
      if (token.startsWith("@")) { // only do something if it's a timing command
        int variance = - 10 + rng.nextInt(11); // vary timing by +/- 10ms
        int timeval = Integer.parseInt(token.substring(1)); // grab original timing value (strip the "@", convert the rest to an integer)
        if (timeval + variance > 0) { // only vary if this will result in events after time 0.  no playing notes befor the song begins!
          token = "@" + (timeval+variance); // create a new token "@XXX" where XXX is the new time value in ms
        }
      }
      newmusicstring = newmusicstring + " " + token; // add it to the end.
    }
    newmusicstring = newmusicstring.trim(); // I'm OCD here... strip excess whitespace
    System.out.println("New: " + newmusicstring); // print out the new sequence, for comparison
    pattern.setMusicString(newmusicstring); // prepare for saving/playing
    player.saveMidi(pattern, new File(filename + ".NEW.midi")); // save to new file
  
    System.exit(0); // exit gracefully, allowing JFugue to release the MIDI interfaces
  }
}
